local lg = love.graphics

-- Class: display
local display = {}
local mode = {}
local function round(num, idp)
    local mult = 10^(idp or 0)
    return math.floor(num * mult + 0.5) / mult
end

-- Property: debug
-- Defaults to false
display.debug = false

-- Method: getDisplays
-- Return: A table with display ids
function display.getDisplays()
    local count = love.window.getDisplayCount()
    local displays = {}
    for i=1, count do
        displays[i] = i
    end
    return displays
end

-- Method: getAspectRatio
-- Arguments:
--  width (int) - width of display
--  height (int) - height of display
-- Return: A table with aspect ratio in width, height, decimal
function display.calculateAspectRatio(width, height)
    local aspect_ratio = {}
    aspect_ratio.decimal = round(width/height, 2)

    local rem
    local newW = width
    local newH = height

    while (height > 0) do
        rem = width % height
        width = height
        height = rem
    end

    newH = newH / width
    newW = newW / width

    aspect_ratio.width = newW
    aspect_ratio.height = newH

    return aspect_ratio
end

-- Method: getAspectRatio
-- Arguments:
--  display_id (bool) - Optionally specify a display or default to current.
-- Return: A table with aspect ratio in width, height, decimal
function display.getAspectRatio(display_id)
    local display_id = display_id or display.getCurrent()
    local width, height = love.window.getDesktopDimensions(display_id)
    local aspect_ratio = display.calculateAspectRatio(width, height)

    return aspect_ratio
end

-- Method: getCurrent
-- Return: Display id, width, height
function display.getCurrent()
    local _, _, flags = love.window.getMode()
    local width, height = love.window.getDesktopDimensions(flags.display)
    if display.debug then
        print('display:' .. flags.display, width .. ' x ' .. height, '\n')
    end
    return flags.display, width, height
end

-- Method: getFullscreenModes
-- Arguments:
--  display_id (bool) - Optionally specify a display or default to current.
-- Return: Available modes for the current display.
function display.getFullscreenModes(display_id)
    local display_id = display_id or display.getCurrent()
    local modes = love.window.getFullscreenModes(display_id)

    -- sort from smallest to largest
    table.sort(modes, function(a, b)
        return a.width*a.height < b.width*b.height
    end)

    if display.debug then
        for i, m in pairs(modes) do
            print(i, m.width, m.height)
        end
        print('\n')
    end

    return modes
end

-- Method: setScreen
-- Configure display options.
--
-- Arguments:
--  width (int) - screen width
--  height (int) - screen height
--  game_width (int) - game width
--  game_height (int) - game height
--  stretch (bool) - scale to screen dimensions, overrides scale
--  scale (bool) - true: use integer scaling, false: only maintain aspect ratio
--  flags (table) - love.window.setMode flags
function display.setScreen(width, height, game_width, game_height, stretch, scale, flags)
    -- Set display
    flags.display = display.getCurrent()
    love.window.setMode(width, height, flags)

    -- Handle scaling
    local hs, ws
    if scale then
        -- Integer scaling
        ws = math.floor(width/game_width)
        hs = math.floor(height/game_height)

        -- Maintain aspect ratio if we integer scale is less than 1
        if ws < 1 or hs < 1 then
            ws = width/game_width
            hs = height/game_height
        end

        -- Scale by equal amounts
        if ws > hs then ws = hs else hs = ws end
    else
        -- Scale to max
        ws = width/game_width
        hs = height/game_height

        -- Scale by equal amounts
        if ws > hs then ws = hs else hs = ws end
    end

    -- Fill screen
    if stretch then
        ws = width/game_width
        hs = height/game_height
    end

    -- Set mode
    mode = {
        width = width,
        height = height,
        game_width = game_width,
        game_height = game_height,
        scale_width = ws,
        scale_height = hs,
        flags = flags,
        scale = false,
        stretch = false,
    }
    if scale then mode.scale = true end
    if stretch then mode.stretch = true mode.scale = false end

end

-- Method: getModes
-- Arguments:
--  m (str) - Optionally get a specific value
-- Return: internal settings table or value
function display.getModes(m)
    if m then
        return mode[m]
    else
        return mode
    end
end

-- Method: set
-- Use at the beginning of love.draw.
function display.set()
    lg.push()

    local sw = mode.game_width*mode.scale_width
    local sh = mode.game_height*mode.scale_height
    local dx = (mode.width - sw) * 0.5
    local dy = (mode.height - sh) * 0.5

    lg.translate(dx, dy)
    lg.scale(mode.scale_width, mode.scale_height)
    lg.setScissor(dx, dy, sw, sh)
end

-- Method: unset
-- Use at the end of love.draw.
function display.unset()
    lg.pop()
end

return display
