local display = require 'display'
require 'loveframes'

local lg = love.graphics
local lt = love.timer
local lw = love.window

local loveframes_tabs, loveframes_opts

local options = {
    mouse_x = love.mouse.getX(),
    mouse_y = love.mouse.getY(),
    displays = {},
    resolutions = {},
    preferences = {
        fsaa = 0,
        stretch = false,
        scale = false,
        height = 0,
        width = 0,
        fullscreen = false,
        fullscreentype = 'normal',
        vsync = false,
    }
}


function options:load()
    local font = lg.newFont(14)
    lg.setFont(font)

    local width, height, flags = lw.getMode( )
    local _, fstype = love.window.getFullscreen( )
    -- Fill preferences
    options.preferences.fsaa = flags.fsaa
    options.preferences.stretch = display.getModes('stretch')
    options.preferences.scale = display.getModes('scale')
    options.preferences.height = height
    options.preferences.width = width
    options.preferences.fullscreen = flags.fullscreen
    options.preferences.fullscreentype = fstype
    options.preferences.vsync = flags.vsync

    -- Table of available resolutions
    local all_resolutions = {}

    options.displays = display.getDisplays()
    for i,v in ipairs(options.displays) do
        options.resolutions[i] = {
            correct = {},
            supported = {},
        }
        local modes = display.getFullscreenModes(v)
        local ar = display.getAspectRatio(v)
        print('\ndisplay: ' .. v, ar.width..':'..ar.height .. '  ' .. ar.decimal)
        print('-----------------------')
        print('resolutions with correct aspect ratio')
        print('-----------------------')
        for kk,vv in pairs(modes) do
            local dar = display.calculateAspectRatio(vv.width,vv.height)
            if dar.decimal == ar.decimal then
                print(kk, vv.width, vv.height)
                local tab = {width = vv.width, height = vv.height}
                table.insert(all_resolutions, tab)
                table.insert(options.resolutions[i].correct, tab)
            else
                local tab = {width = vv.width, height = vv.height}
                table.insert(all_resolutions, tab)
                table.insert(options.resolutions[i].supported, tab)
            end
        end

        print('-----------------------')
        print('supported resolutions')
        print('-----------------------')
        for kk,vv in pairs(modes) do
            print(kk, vv.width, vv.height)
        end
    end

    -- Setup resolution table
    table.sort(all_resolutions,function(a,b)
      return a.width < b.width
    end)

    local r = {}
    for k,v in pairs(all_resolutions) do
        table.insert(r, v.width .. 'x' ..v.height)
    end
    all_resolutions = r

    -- make unique keys
    local hash = {}
    for _,v in ipairs(all_resolutions) do
        hash[v] = true
    end

    -- transform keys back into values
    local res = {}
    for k,_ in pairs(hash) do
        res[#res+1] = k
    end

    all_resolutions = res

    -- LOVE FRAMES
    local frame = loveframes.Create("frame")
    frame:SetName("Options")
    frame:SetSize(500, 400)
    frame:SetPos(10,40)
    frame:ShowCloseButton(false)

    loveframes_opts(frame, options.preferences, all_resolutions)
    loveframes_tabs(frame)
end

function options:draw()
    lg.setColor(255,255,255)
    local width, height = lw.getDimensions()
    local display = 'Display: ' .. display.getCurrent()
    local dimensions = ' / ' .. width .. 'x' .. height
    local fps = ' / FPS: ' .. lt.getFPS()
    local str = display .. dimensions .. fps
    lg.print(str, 10, 10)

    loveframes.draw()
end

function options:update(dt)
    loveframes.update(dt)
end

function options:mousepressed(x, y, button)
    loveframes.mousepressed(x, y, button)
end
function options:mousereleased(x, y, button)
    loveframes.mousereleased(x, y, button)
end
function options:keypressed(key, unicode)
    loveframes.keypressed(key, unicode)
end
function options:keyreleased(key)
    loveframes.keyreleased(key)
end
function options:textinput(text)
    loveframes.textinput(text)
end


function loveframes_tabs(frame)

    -- LOVE FRAME TABS
    local tabs = loveframes.Create("tabs", frame)
    tabs:SetPos(245, 30)
    tabs:SetSize(250, 365)

    for i=1, #options.displays do
        local panel = loveframes.Create("panel")
        panel.Draw = function()
            -- print('panels',i)
        end
        local ar = display.getAspectRatio(i)
        local width, height = love.window.getDesktopDimensions(i)

        local text1 = loveframes.Create("text", panel)

        local txt = 'Display: ' .. i .. ' \n (' .. width .. 'x' .. height .. ') '
        txt = txt .. ar.width .. ':' .. ar.height .. ' ' .. ar.decimal
        txt = txt .. ' \n \n Correct aspect ratio \n \n '
        for k,v in pairs(options.resolutions[i].correct) do
            txt = txt .. '['..k..'] ' .. v.width .. ' x ' .. v.height .. ' \n '
        end
        txt = txt .. ' \n All supported resolutions \n \n '
        for k,v in pairs(options.resolutions[i].supported) do
            txt = txt .. '['..k..'] ' .. v.width .. ' x ' .. v.height .. ' \n '
        end
        text1:SetText(txt)
        -- text1:SetAlwaysUpdate(true)
        -- text1.Update = function(object, dt)
        --     object:Center()
        -- end
        tabs:AddTab("Display " .. i, panel, "Display " .. i)
        -- tabs:AddScrollButtons(true)
    end
end
function loveframes_opts(frame, preferences, all_resolutions)

    -- //////////////////////////////////////////////////////
    local panel = loveframes.Create("panel", frame)
    panel:SetPos(5, 30)
    panel:SetSize(230, 35)

    local checkbox1 = loveframes.Create("checkbox", panel)
    checkbox1:SetText('fullscreen')
    checkbox1:SetPos(10, 8)
    if preferences.fullscreen then
        checkbox1:SetChecked(true)
    end
    checkbox1.Update = function(object, dt)
        if checkbox1:GetChecked() ~= preferences.fullscreen then
            preferences.fullscreen = checkbox1:GetChecked()
            print('fullscreen:', preferences.fullscreen)
        end
    end

    local checkbox2 = loveframes.Create("checkbox", panel)
    checkbox2:SetText('vsync')
    checkbox2:SetPos(125, 8)
    if preferences.vsync then
        checkbox2:SetChecked(true)
    end
    checkbox2.Update = function(object, dt)
        if checkbox2:GetChecked() ~= preferences.vsync then
            preferences.vsync = checkbox2:GetChecked()
            print('vsync:', preferences.vsync)
        end
    end

    -- //////////////////////////////////////////////////////
    local panel1 = loveframes.Create("panel", frame)
    panel1:SetPos(5, 70)
    panel1:SetSize(230, 35)

    local text2 = loveframes.Create("text", panel1)
    text2:SetText('fsaa')
    text2:SetPos(55, 10)

    local numberbox = loveframes.Create("numberbox", panel1)
    numberbox:SetPos(5, 5)
    numberbox:SetSize(45, 25)
    numberbox:SetMin(0)
    if preferences.fsaa then
        numberbox:SetValue(preferences.fsaa)
    end
    numberbox.Update = function(object, dt)
        if numberbox:GetValue() ~= preferences.fsaa then
            preferences.fsaa = numberbox:GetValue()
            print('fsaa:', preferences.fsaa)
        end
    end

    -- //////////////////////////////////////////////////////
    local panel2 = loveframes.Create("panel", frame)
    panel2:SetPos(5, 110)
    panel2:SetSize(230, 35)

    local text3 = loveframes.Create("text", panel2)
    text3:SetText('fullsceen type')
    text3:SetPos(135, 10)

    local multichoice = loveframes.Create("multichoice", panel2)
    multichoice:SetPos(5, 5)
    multichoice:SetSize(125, 25)
    multichoice:AddChoice('desktop')
    multichoice:AddChoice('normal')
    if preferences.fullscreentype then
        multichoice:SetChoice(preferences.fullscreentype)
    end
    multichoice.Update = function(object, dt)
        if multichoice:GetChoice() ~= preferences.fullscreentype then
            preferences.fullscreentype = multichoice:GetChoice()
            print('fullscreentype:', preferences.fullscreentype)
        end
    end

    -- //////////////////////////////////////////////////////
    local panel3 = loveframes.Create("panel", frame)
    panel3:SetPos(5, 150)
    panel3:SetSize(230, 35)

    local checkbox3, checkbox4
    checkbox3 = loveframes.Create("checkbox", panel3)
    checkbox3:SetText('integer scale')
    checkbox3:SetPos(10, 8)
    if preferences.scale then
        checkbox3:SetChecked(true)
    end
    checkbox3.Update = function(object, dt)
        if checkbox3:GetChecked() ~= preferences.scale then
            preferences.scale = checkbox3:GetChecked()
            print('integer scale:', preferences.scale)
            if checkbox3:GetChecked() then
                checkbox4:SetChecked(false)
            end
        end
    end

    checkbox4 = loveframes.Create("checkbox", panel3)
    checkbox4:SetText('stretch')
    checkbox4:SetPos(125, 8)
    if preferences.stretch then
        checkbox4:SetChecked(true)
    end
    checkbox4.Update = function(object, dt)
        if checkbox4:GetChecked() ~= preferences.stretch then
            preferences.stretch = checkbox4:GetChecked()
            print('stretch:', preferences.stretch)
            if checkbox4:GetChecked() then
                checkbox3:SetChecked(false)
            end
        end
    end

    -- //////////////////////////////////////////////////////
    local panel4 = loveframes.Create("panel", frame)
    panel4:SetPos(5, 190)
    panel4:SetSize(230, 135)

    local text4 = loveframes.Create("text", panel4)
    text4:SetText('resolution')
    text4:SetPos(5, 5)

    local numberbox1, numberbox2
    local multichoice1 = loveframes.Create("multichoice", panel4)
    multichoice1:SetPos(5, 30)
    for k,v in pairs(all_resolutions) do
        multichoice1:AddChoice(v)
    end
    multichoice1.OnChoiceSelected = function(object, choice)
        function string:split(sep)
            local sep, fields = sep or ":", {}
            local pattern = string.format("([^%s]+)", sep)
            self:gsub(pattern, function(c) fields[#fields+1] = c end)
            return fields
        end

        local resolution = multichoice1:GetChoice()
        local resolution = resolution:split("x")

        preferences.width = resolution[1]
        preferences.height = resolution[2]

        w,h = preferences.width, preferences.height

        numberbox1:SetValue(preferences.width)
        numberbox2:SetValue(preferences.height)
    end

    numberbox1 = loveframes.Create("numberbox", panel4)
    numberbox1:SetPos(5, 65)
    numberbox1:SetSize(85, 25)
    numberbox1:SetMin(0)
    numberbox1:SetMax(10000000000000)
    if preferences.width then
        numberbox1:SetValue(preferences.width)
    end

    numberbox2 = loveframes.Create("numberbox", panel4)
    numberbox2:SetPos(115, 65)
    numberbox2:SetSize(85, 25)
    numberbox2:SetMin(0)
    numberbox2:SetMax(10000000000000)
    if preferences.height then
        numberbox2:SetValue(preferences.height)
    end

    local text5 = loveframes.Create("text", panel4)
    text5:SetText('X')
    text5:SetPos(98, 70)

    local button = loveframes.Create("button", panel4)
    button:SetWidth(220)
    button:SetY(100)
    button:CenterX(true)
    button:SetText("Apply Changes")
    button.OnClick = function(object, x, y)

        local w,h,gw,h,st,sc,flags
        gw, gh = display.getModes('game_width'), display.getModes('game_height')
        st, sc = preferences.stretch, preferences.scale
        flags = {
            fullscreen = preferences.fullscreen,
            fullscreentype = preferences.fullscreentype,
            fsaa = preferences.fsaa,
            vsync = preferences.vsync,
        }

        preferences.width = numberbox1:GetValue()
        preferences.height = numberbox2:GetValue()

        w,h = preferences.width, preferences.height
        print('resolution set: ' .. w ..'x'..h)
        display.setScreen(w,h,gw,gh,st,sc,flags)
    end

end

return options
